@@ -1,10 +1,10 @@
 /*
     fileio
-    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,23 +13,20 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "fileio.h"
-#include <time.h>
 #ifdef HAVE_UTIME_H
 # include <utime.h>
 #endif /* HAVE_UTIME_H */
 #if defined (TM_IN_SYS_TIME) && defined (HAVE_SYS_TIME_H)
 # include <sys/time.h>
 #endif /* defined (TM_IN_SYS_TIME) && defined (HAVE_SYS_TIME_H) */
-#if GLIB_CHECK_VERSION(2,6,0)
-# include <glib/gstdio.h>
-#endif /* GLIB_CHECK_VERSION(2,6,0) */
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32)
 # include <shlwapi.h>
-#endif /* G_OS_WIN32 */
+#elif GLIB_CHECK_VERSION(2,16,0)
+# include <gio/gio.h>
+#endif /* GLIB_CHECK_VERSION(2,16,0) */
 
 
 struct _FileIO
@@ -66,7 +63,7 @@ fileio_get_absolute_path (const gchar *file,
 # if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR p, lpszFile;
   TCHAR szFile[MAX_PATH];   /* ja:フルパス */
-  TCHAR lpszLongName[MAX_PATH];
+  TCHAR szLongName[MAX_PATH];
 # endif /* not GLIB_CHECK_VERSION(2,6,0) */
 
   if (!file)
@@ -168,7 +165,7 @@ fileio_get_absolute_path (const gchar *file,
                 FindClose (hFind);
               else
                 lstrcpyW (rcf.cFileName, L".");
-              if (lstrcmpW (rcf.cFileName, L"."))
+              if (lstrcmpW (rcf.cFileName, L".") != 0)
                 {
                   g_memmove (szLongName + s, rcf.cFileName,
                                     lstrlenW (rcf.cFileName) * sizeof (WCHAR));
@@ -195,29 +192,15 @@ fileio_get_absolute_path (const gchar *file,
       CHAR szFile[MAX_PATH];   /* ja:フルパス */
       CHAR szLongName[MAX_PATH];
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
-
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
+
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       if (PathIsRelativeA (lpszFile))
         {
           LPSTR lpszPath = NULL, lpszTemp;
 
           if (path)
-            {
-#  if GLIB_CHECK_VERSION(2,8,0)
-              lpszPath = g_win32_locale_filename_from_utf8 (path);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-              lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-              lpszPath = g_strdup (path);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
-            }
+            lpszPath = g_win32_locale_filename_from_utf8 (path);
           if (!lpszPath || PathIsRelativeA (lpszPath))
             {
               g_free (lpszPath);
@@ -279,7 +262,7 @@ fileio_get_absolute_path (const gchar *file,
                 FindClose (hFind);
               else
                 lstrcpyA (rcf.cFileName, ".");
-              if (lstrcmpA (rcf.cFileName, "."))
+              if (lstrcmpA (rcf.cFileName, ".") != 0)
                 {
                   g_memmove (szLongName + s, rcf.cFileName,
                                     lstrlenA (rcf.cFileName) * sizeof (CHAR));
@@ -363,7 +346,7 @@ fileio_get_absolute_path (const gchar *file,
   while (i < leng)
     if (G_IS_DIR_SEPARATOR (absolate[i]) && absolate[i + 1] == '.'
         && absolate[i + 2] == '.'
-        && (G_IS_DIR_SEPARATOR (absolate[i + 3])  || absolate[i + 3] == '\0'))
+        && (G_IS_DIR_SEPARATOR (absolate[i + 3]) || absolate[i + 3] == '\0'))
       {
         gint j;
 
@@ -509,8 +492,7 @@ fileio_open (const gchar *file,
 # endif /* not GLIB_CHECK_VERSION(2,6,0) */
 #else /* not G_OS_WIN32 */
   int fd, flags;
-  gint i;
-  guint length;
+  GList *glist;
 #endif /* not G_OS_WIN32 */
 
   if (!file)
@@ -560,13 +542,7 @@ fileio_open (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       hFile = CreateFileA (lpszFile, access, share, NULL, mode,
                                                 FILE_ATTRIBUTE_NORMAL, NULL);
       g_free (lpszFile);
@@ -614,25 +590,21 @@ fileio_open (const gchar *file,
   G_LOCK (critical);
   critical = TRUE;
 # endif /* USE_THREAD */
-  length = g_list_length (glist_fileio);
-  for (i = 0; i < length; i++)
-    {
-      fio = g_list_nth_data (glist_fileio, i);
-      if (g_strfilecmp (fio->file, path) == 0
-                                            && (fio->share & access) != access)
-        break;
-    }
+  for (glist = g_list_first (glist_fileio); glist; glist = g_list_next (glist))
+    if (g_strfilecmp (((FileIO *)glist->data)->file, path) == 0
+                        && (((FileIO *)glist->data)->share & access) != access)
+      break;
 # ifdef USE_THREAD
   critical = FALSE;
   G_UNLOCK (critical);
 # endif /* USE_THREAD */
-  if (i < length)
+  if (glist)
     {
       g_free (path);
       return NULL;
     }
 
-  fd = open (path, flags,
+  fd = g_open (path, flags,
                     S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
   if (fd == -1)
     {
@@ -646,7 +618,7 @@ fileio_open (const gchar *file,
 # endif /* HAVE_FLOCK */
         (mode == FILEIO_MODE_CREATE_ALWAYS && (
 # ifdef HAVE_FTRUNCATE
-                    ftruncate (fd, 0) || 
+                    ftruncate (fd, 0) ||
 # endif /* HAVE_FTRUNCATE */
                     lseek (fd, 0, SEEK_SET) == -1)))
     {
@@ -916,7 +888,7 @@ fileio_eof (FileIO *fio)
   critical = TRUE;
 #  endif /* USE_THREAD */
   result = fio && g_list_find (glist_fileio, fio)
-                                    ? ftruncate (fio->fd, offset) == 0: FALSE;
+                                    ? ftruncate (fio->fd, offset) == 0 : FALSE;
 #  ifdef USE_THREAD
   critical = FALSE;
   G_UNLOCK (critical);
@@ -1143,13 +1115,7 @@ fileio_mkdir (const gchar *dir)
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszPath = g_win32_locale_filename_from_utf8 (path);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszPath = g_strdup (path);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       result = CreateDirectoryA (lpszPath, NULL);
       g_free (lpszPath);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
@@ -1157,13 +1123,8 @@ fileio_mkdir (const gchar *dir)
     }
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 #else /* not G_OS_WIN32 */
-# if GLIB_CHECK_VERSION(2,6,0)
   result = g_mkdir (path, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP
                                                     | S_IROTH | S_IXOTH) == 0;
-# else /* not GLIB_CHECK_VERSION(2,6,0) */
-  result = mkdir (path, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP
-                                                    | S_IROTH | S_IXOTH) == 0;
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
 #endif /* not G_OS_WIN32 */
   g_free (path);
   return result;
@@ -1226,13 +1187,7 @@ fileio_remove (const gchar *path)
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
               lpszPath = g_win32_locale_filename_from_utf8 (path);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-              lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-              lpszPath = g_strdup (path);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
               result = RemoveDirectoryA (lpszPath);
               g_free (lpszPath);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
@@ -1240,11 +1195,7 @@ fileio_remove (const gchar *path)
             }
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 #else /* not G_OS_WIN32 */
-# if GLIB_CHECK_VERSION(2,6,0)
           result = g_rmdir (path) == 0;
-# else /* not GLIB_CHECK_VERSION(2,6,0) */
-          result = rmdir (path) == 0;
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
 #endif /* not G_OS_WIN32 */
         }
     }
@@ -1279,13 +1230,7 @@ fileio_remove (const gchar *path)
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
           lpszPath = g_win32_locale_filename_from_utf8 (path);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-          lpszPath = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-          lpszPath = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
           result = DeleteFileA (lpszPath);
           g_free (lpszPath);
 # endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
@@ -1293,11 +1238,7 @@ fileio_remove (const gchar *path)
         }
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 #else /* not G_OS_WIN32 */
-# if GLIB_CHECK_VERSION(2,6,0)
       result = g_remove (path) == 0;
-# else /* not GLIB_CHECK_VERSION(2,6,0) */
-      result = remove (path) == 0;
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
 #endif /* not G_OS_WIN32 */
     }
   return result;
@@ -1369,13 +1310,7 @@ fileio_gettime (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || !defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       if (!lpszFile)
         return FALSE;
       hFind = FindFirstFileA (lpszFile, &rcFind);
@@ -1391,7 +1326,7 @@ fileio_gettime (const gchar *file,
     }
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 #else /* not G_OS_WIN32 */
-  if (stat (file, &buf))
+  if (g_stat (file, &buf))
     return FALSE;
   if (atime)
     *atime = buf.st_atime;
@@ -1454,13 +1389,7 @@ fileio_settime (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,6,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
       hFile = CreateFileA (lpszFile, GENERIC_WRITE,
                                 FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
                                 OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
@@ -1479,7 +1408,7 @@ fileio_settime (const gchar *file,
   return CloseHandle (hFile);
 #else /* not G_OS_WIN32 */
 # ifdef HAVE_UTIME
-  if (stat (file, &sbuf))
+  if (g_stat (file, &sbuf))
     return FALSE;
   ubuf.actime  = atime ? *atime : sbuf.st_atime;
   ubuf.modtime = mtime ? *mtime : sbuf.st_mtime;
@@ -1565,6 +1494,7 @@ fileio_setsystime (FileIOTime    *ftime,
 * ja:応用ファイル関数群                                                       *
 *                                                                             *
 ******************************************************************************/
+#ifndef USE_GTK_EMULATE
 /*  ja:ファイルのアイコンを取得する
          file,ファイル名
     icon_size,アイコンサイズ
@@ -1573,53 +1503,42 @@ GdkPixbuf *
 fileio_extract_icon (const gchar *file,
                      GtkIconSize  icon_size)
 {
-#if defined (G_OS_WIN32)
+#if defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
   gint icon_x, icon_y;
   GdkPixbuf *pixbuf = NULL;
+# ifdef G_OS_WIN32
   HICON hIcon = NULL;
   ICONINFO ii;
   UINT uFlags;
-# if GLIB_CHECK_VERSION(2,2,0)
-  GtkSettings *settings;
-# endif /* GLIB_CHECK_VERSION(2,2,0) */
-# if ! GLIB_CHECK_VERSION(2,6,0)
+#  if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR lpszFile;
   SHFILEINFO shfi;
-#  ifdef UNICODE
+#   ifdef UNICODE
   gchar *utf8str;
-#  endif /* UNICODE */
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+#   endif /* UNICODE */
+#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+# endif /* G_OS_WIN32 */
 
-  if (!file)
+  if (!gtk_icon_size_lookup (icon_size, &icon_x, &icon_y))
+    switch (icon_size)
+      {
+        case GTK_ICON_SIZE_MENU:          icon_x = 16; icon_y = 16; break;
+        case GTK_ICON_SIZE_SMALL_TOOLBAR: icon_x = 18; icon_y = 18; break;
+        case GTK_ICON_SIZE_LARGE_TOOLBAR: icon_x = 24; icon_y = 24; break;
+        case GTK_ICON_SIZE_BUTTON:        icon_x = 20; icon_y = 20; break;
+        case GTK_ICON_SIZE_DND:           icon_x = 32; icon_y = 32; break;
+        case GTK_ICON_SIZE_DIALOG:        icon_x = 48; icon_y = 48; break;
+        default:                          icon_x =  0; icon_y =  0;
+      }
+  if (!file || icon_x <= 0 || icon_y <= 0)
+#endif /* defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
     return NULL;
-  switch (icon_size)
-    {
-      case GTK_ICON_SIZE_MENU:          icon_x = 16; icon_y = 16; break;
-      case GTK_ICON_SIZE_SMALL_TOOLBAR: icon_x = 18; icon_y = 18; break;
-      case GTK_ICON_SIZE_LARGE_TOOLBAR: icon_x = 24; icon_y = 24; break;
-      case GTK_ICON_SIZE_BUTTON:        icon_x = 20; icon_y = 20; break;
-      case GTK_ICON_SIZE_DND:           icon_x = 32; icon_y = 32; break;
-      case GTK_ICON_SIZE_DIALOG:        icon_x = 48; icon_y = 48; break;
-      default:                          icon_x =  0; icon_y =  0;
-    }
-# if GLIB_CHECK_VERSION(2,2,0)
-  settings = gtk_settings_get_default ();
-  if (settings)
-    {
-      gint w, h;
-
-      if (gtk_icon_size_lookup_for_settings (settings, icon_size, &w, &h))
-        {
-          icon_x = w;
-          icon_y = h;
-        }
-    }
+#if defined (G_OS_WIN32)
   uFlags = ABS (GetSystemMetrics (SM_CXSMICON) - icon_x)
                                 + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
                                 < ABS (GetSystemMetrics (SM_CXICON) - icon_x)
                                 + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
                                         ? SHGFI_SMALLICON : SHGFI_LARGEICON;
-# endif /* GLIB_CHECK_VERSION(2,2,0) */
 # if GLIB_CHECK_VERSION(2,6,0)
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
@@ -1647,13 +1566,7 @@ fileio_extract_icon (const gchar *file,
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
 
 # if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-#  if GLIB_CHECK_VERSION(2,8,0)
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-#  elif GLIB_CHECK_VERSION(2,6,0)
-      lpszFile = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
-#  else /* not GLIB_CHECK_VERSION(2,8,0) */
-      lpszFile = g_strdup (file);
-#  endif /* not GLIB_CHECK_VERSION(2,8,0) */
       if ((BOOL)SHGetFileInfoA (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
                                 SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags))
         hIcon = shfi.hIcon;
@@ -1728,12 +1641,7 @@ fileio_extract_icon (const gchar *file,
     }
   if (hIcon)
     DestroyIcon (hIcon);
-  return pixbuf;
-#elif GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
-  GdkPixbuf *pixbuf = NULL;
-
-  if (!file)
-    return NULL;
+#elif GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
   if (g_file_test (file, G_FILE_TEST_IS_DIR))
     {
       const gchar *home;
@@ -1752,139 +1660,120 @@ fileio_extract_icon (const gchar *file,
     }
   else
     {
-      gchar *text;
-      gssize length;
+      gchar *mime_type = NULL;
+# if GLIB_CHECK_VERSION(2,16,0)
+      gboolean uncertain;
+      gchar *content_type;
 
-      text = fileio_load (MIMEFILE, &length);
-      if (text)
+      content_type = g_content_type_guess (file, NULL, 0, &uncertain);
+      if (content_type)
         {
-          gchar **mimes, *utf8str, *mime_type = NULL;
-          gint i;
-
-          text = g_realloc (text, length + 1);
-          text[length] = '\0';
-          utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-          mimes = g_strsplit (text, "\n", 0);
-          for (i = 0; mimes[i]; i++)
+          mime_type = g_content_type_get_mime_type (content_type);
+          g_free (content_type);
+        }
+# endif /* GLIB_CHECK_VERSION(2,16,0) */
+# ifdef MIMEFILE
+#  if GLIB_CHECK_VERSION(2,16,0)
+      if (!mime_type)
+        {
+#  endif /* GLIB_CHECK_VERSION(2,16,0) */
+          gchar *text;
+          gssize length;
+          text = fileio_load (MIMEFILE, &length);
+          if (text)
             {
-              gchar **globs;
+              gchar **mimes, *utf8str;
+              gint i;
 
-              globs = g_strsplit (mimes[i], G_SEARCHPATH_SEPARATOR_S, 0);
-              if (globs[0])
+              text = g_realloc (text, length + 1);
+              text[length] = '\0';
+              utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+              mimes = g_strsplit (text, "\n", 0);
+              for (i = 0; mimes[i]; i++)
                 {
-                  gint j;
+                  gchar **globs;
 
-                  for (j = 1; globs[j]; j++)
-                    if (g_pattern_match_simple (globs[j], utf8str))
-                      {
-                        mime_type = g_strdup (globs[0]);
-                        break;
-                      }
-                }
-              g_strfreev (globs);
-              if (mime_type)
-                break;
-            }
-          g_strfreev (mimes);
-          g_free (utf8str);
-          g_free (text);
-          if (mime_type)
-            {
-              gchar *icon_name, *separator;
-
-              icon_name = g_strconcat ("gnome-mime-", mime_type, NULL);
-              g_free (mime_type);
-              separator = g_strchr (icon_name, '/');
-              if (separator)
-                {
-                  GtkIconTheme *icon_theme;
-
-                  *separator = '-';
-                  icon_theme = gtk_icon_theme_get_default ();
-                  if (icon_theme)
+                  globs = g_strsplit (mimes[i], G_SEARCHPATH_SEPARATOR_S, 0);
+                  if (globs[0])
                     {
-                      gboolean result;
+                      gint j;
 
+                      for (j = 1; globs[j]; j++)
+                        if (g_pattern_match_simple (globs[j], utf8str))
+                          {
+                            mime_type = g_strdup (globs[0]);
+                            break;
+                          }
+                    }
+                  g_strfreev (globs);
+                  if (mime_type)
+                    break;
+                }
+              g_strfreev (mimes);
+              g_free (utf8str);
+              g_free (text);
+            }
+#  if GLIB_CHECK_VERSION(2,16,0)
+        }
+#  endif /* GLIB_CHECK_VERSION(2,16,0) */
+# endif /* MIMEFILE */
+      if (mime_type)
+        {
+          gchar *icon_name, *separator;
+
+          icon_name = g_strconcat ("gnome-mime-", mime_type, NULL);
+          g_free (mime_type);
+          separator = g_strchr (icon_name, '/');
+          if (separator)
+            {
+              GtkIconTheme *icon_theme;
+
+              *separator = '-';
+              icon_theme = gtk_icon_theme_get_default ();
+              if (icon_theme)
+                {
+                  gboolean result;
+
+                  result = gtk_icon_theme_has_icon (icon_theme, icon_name);
+                  if (!result)
+                    {
+                      *separator = '\0';
                       result = gtk_icon_theme_has_icon (icon_theme, icon_name);
-                      if (!result)
-                        {
-                          *separator = '\0';
-                          result = gtk_icon_theme_has_icon (icon_theme,
-                                                                    icon_name);
-                        }
-                      if (result)
-                        {
-                          gint icon_x, icon_y;
+                    }
+                  if (result)
+                    {
 # if GLIB_CHECK_VERSION(2,6,0)
-                          gint *sizes;
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
-                          GtkSettings *settings;
+                      gint *sizes;
 
-                          switch (icon_size)
-                            {
-                              case GTK_ICON_SIZE_MENU:
-                                icon_x = icon_y = 16;
-                                break;
-                              case GTK_ICON_SIZE_SMALL_TOOLBAR:
-                                icon_x = icon_y = 18;
-                                break;
-                              case GTK_ICON_SIZE_LARGE_TOOLBAR:
-                                icon_x = icon_y = 24;
-                                break;
-                              case GTK_ICON_SIZE_BUTTON:
-                                icon_x = icon_y = 20;
-                                break;
-                              case GTK_ICON_SIZE_DND:
-                                icon_x = icon_y = 32;
-                                break;
-                              case GTK_ICON_SIZE_DIALOG:
-                                icon_x = icon_y = 48;
-                                break;
-                              default:
-                                icon_x = icon_y = 0;
-                            }
-                          settings = gtk_settings_get_default ();
-                          if (settings)
-                            {
-                              gint w, h;
-
-                              if (gtk_icon_size_lookup_for_settings (settings,
-                                                            icon_size, &w, &h))
-                                {
-                                  icon_x = w;
-                                  icon_y = h;
-                                }
-                            }
-# if GLIB_CHECK_VERSION(2,6,0)
-                          sizes = gtk_icon_theme_get_icon_sizes (icon_theme,
+                      sizes = gtk_icon_theme_get_icon_sizes (icon_theme,
                                                                     icon_name);
-                          if (sizes)
-                            {
-                              gint n = 0;
+                      if (sizes)
+                        {
+                          gint i, n = 0;
 
-                              for (i = 0; sizes[i] > 0; i++)
-                                if (ABS (icon_x - sizes[i])
+                          for (i = 0; sizes[i] > 0; i++)
+                            if (ABS (icon_x - sizes[i])
                                                     + ABS (icon_y - sizes[i])
                                         < ABS (icon_x - n) + ABS (icon_y - n))
-                                  n = sizes[i];
-                              g_free (sizes);
-                              if (n > 0)
-                                pixbuf = gtk_icon_theme_load_icon (icon_theme,
-                                                        icon_name, n, 0, NULL);
-                            }
-                          if (!pixbuf)
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
+                              n = sizes[i];
+                          g_free (sizes);
+                          if (n > 0)
                             pixbuf = gtk_icon_theme_load_icon (icon_theme,
-                                    icon_name, (icon_x + icon_y) / 2, 0, NULL);
+                                                        icon_name, n, 0, NULL);
                         }
+                      if (!pixbuf)
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+                        pixbuf = gtk_icon_theme_load_icon (icon_theme,
+                                    icon_name, (icon_x + icon_y) / 2, 0, NULL);
                     }
                 }
-              g_free (icon_name);
             }
+          g_free (icon_name);
         }
     }
+#endif /* GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
+#if defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
   return pixbuf;
-#else /* not G_OS_WIN32 && not MIMEFILE */
-  return NULL;
-#endif /* not G_OS_WIN32 && not MIMEFILE */
+#endif /* defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
 }
+#endif /* not USE_GTK_EMULATE */
