@@ -1,10 +1,10 @@
 /*
     orzcharset
-    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,22 +13,9 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "orzcharset.h"
-#include <gdk/gdkkeysyms.h>
-
-
-static void orz_charset_class_init (OrzCharsetClass *klass);
-static void orz_charset_init       (OrzCharset      *charset);
-
-
-#if GTK_CHECK_VERSION(2,4,0)
-static GtkComboBoxEntryClass *parent_class = NULL;
-#else /* not GTK_CHECK_VERSION(2,4,0) */
-static GtkComboClass *parent_class = NULL;
-#endif /* not GTK_CHECK_VERSION(2,4,0) */
 
 
 enum
@@ -45,44 +32,18 @@ static gint orz_charset_signals[LAST_SIGNAL] = {0};
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
-GtkType
-orz_charset_get_type (void)
-{
-  static GType orz_charset_type = 0;
-
-  if (!orz_charset_type)
-    {
-      const static GTypeInfo charset_info =
-      {
-        sizeof (OrzCharsetClass),
-        NULL,               /* base_init */
-        NULL,               /* base_finalize */
-        (GClassInitFunc)orz_charset_class_init,
-        NULL,               /* class_finalize */
-        NULL,               /* class_data */
-        sizeof (OrzCharset),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc)orz_charset_init,
-      };
-
-#if GTK_CHECK_VERSION(2,4,0)
-      orz_charset_type = g_type_register_static (GTK_TYPE_COMBO_BOX_ENTRY,
-                                            "OrzCharset", &charset_info, 0);
+#if GTK_CHECK_VERSION(2,24,0)
+G_DEFINE_TYPE (OrzCharset, orz_charset, GTK_TYPE_COMBO_BOX_TEXT);
+#elif GTK_CHECK_VERSION(2,4,0)
+G_DEFINE_TYPE (OrzCharset, orz_charset, GTK_TYPE_COMBO_BOX_ENTRY);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
-      orz_charset_type = g_type_register_static (GTK_TYPE_COMBO,
-                                            "OrzCharset", &charset_info, 0);
+G_DEFINE_TYPE (OrzCharset, orz_charset, GTK_TYPE_COMBO);
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
-    }
-
-  return orz_charset_type;
-}
 
 
 static void
 orz_charset_class_init (OrzCharsetClass *klass)
 {
-  parent_class = g_type_class_peek_parent (klass);
-
   klass->activate = NULL;
   klass->edited = NULL;
 
@@ -106,16 +67,16 @@ orz_charset_class_init (OrzCharsetClass *klass)
 
 
 static void
-orz_charset_activate (GtkWidget   *widget,
-                       OrzCharset *charset)
+orz_charset_activate (GtkWidget  *widget,
+                      OrzCharset *charset)
 {
   g_signal_emit_by_name (G_OBJECT (charset), "activate");
 }
 
 
 static void
-orz_charset_changed (GtkWidget   *widget,
-                      OrzCharset *charset)
+orz_charset_changed (GtkWidget  *widget,
+                     OrzCharset *charset)
 {
   g_signal_emit_by_name (G_OBJECT (charset), "edited");
 }
@@ -124,7 +85,7 @@ orz_charset_changed (GtkWidget   *widget,
 #if ! GTK_CHECK_VERSION(2,4,0)
 static void
 orz_charset_grab_focus (GtkWidget *widget,
-                         gpointer   user_data)
+                        gpointer   user_data)
 {
   gtk_widget_grab_focus (GTK_COMBO (widget)->entry);
 }
@@ -132,8 +93,8 @@ orz_charset_grab_focus (GtkWidget *widget,
 
 static gboolean
 orz_charset_mnemonic (GtkWidget *widget,
-                       gboolean   arg1,
-                       gpointer   user_data)
+                      gboolean   arg1,
+                      gpointer   user_data)
 {
   gtk_widget_grab_focus (GTK_COMBO (widget)->entry);
   return TRUE;
@@ -144,7 +105,8 @@ orz_charset_mnemonic (GtkWidget *widget,
 static void
 orz_charset_init (OrzCharset *charset)
 {
-#if GTK_CHECK_VERSION(2,4,0)
+#if GTK_CHECK_VERSION(2,24,0)
+#elif GTK_CHECK_VERSION(2,4,0)
   gtk_combo_box_set_model (GTK_COMBO_BOX (charset),
                     GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING)));
   gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY (charset), 0);
@@ -177,7 +139,22 @@ orz_charset_init (OrzCharset *charset)
 GtkWidget *
 orz_charset_new (void)
 {
+#if GTK_CHECK_VERSION(2,24,0)
+  GtkWidget *charset;
+
+  charset = GTK_WIDGET (g_object_new (ORZ_TYPE_CHARSET,
+                                      "has-entry", TRUE,
+                                      "entry-text-column", 0,
+                                      "id-column", 1,
+                                      NULL));
+  g_signal_connect (G_OBJECT (gtk_bin_get_child (GTK_BIN (charset))),
+                    "activate", G_CALLBACK (orz_charset_activate), charset);
+  g_signal_connect (G_OBJECT (gtk_bin_get_child (GTK_BIN (charset))),
+                    "changed", G_CALLBACK (orz_charset_changed), charset);
+  return charset;
+#else /* not GTK_CHECK_VERSION(2,24,0) */
   return GTK_WIDGET (g_object_new (ORZ_TYPE_CHARSET, NULL));
+#endif /* not GTK_CHECK_VERSION(2,24,0) */
 }
 
 
@@ -225,44 +202,62 @@ gchar *
 orz_charset_get_candidate (OrzCharset *charset)
 {
   gchar *candidate = NULL;
-  gint i;
-#if GTK_CHECK_VERSION(2,4,0)
-  GtkTreeModel *model;
-#else /* not GTK_CHECK_VERSION(2,4,0) */
-  GList *glist;
-#endif /* not GTK_CHECK_VERSION(2,4,0) */
 
-  if (!charset)
-    return NULL;
-#if GTK_CHECK_VERSION(2,4,0)
-  model = gtk_combo_box_get_model (GTK_COMBO_BOX (charset));
-  for (i = gtk_tree_model_iter_n_children (model, NULL) - 1; i >= 0; i--)
+  if (charset)
     {
-      gchar *str, *tmp;
-      GtkTreeIter iter;
+#if GTK_CHECK_VERSION(2,4,0)
+      gint i, column;
+      GtkTreeModel *model;
 
-      gtk_tree_model_iter_nth_child (model, &iter, NULL, i);
-      gtk_tree_model_get (model, &iter, 0, &tmp, -1);
-      str = g_strconcat (tmp, ",", candidate, NULL);
-      g_free (tmp);
-      g_free (candidate);
-      candidate = str;
-    }
+      model = gtk_combo_box_get_model (GTK_COMBO_BOX (charset));
+# if GTK_CHECK_VERSION(2,24,0)
+      column = gtk_combo_box_get_entry_text_column (GTK_COMBO_BOX (charset));
+# else /* not GTK_CHECK_VERSION(2,24,0) */
+      for (column = gtk_tree_model_get_n_columns (model) - 1;
+                                                        column > 0; column--)
+        if (gtk_tree_model_get_column_type (model, column) == G_TYPE_STRING)
+          break;
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
+      for (i = gtk_tree_model_iter_n_children (model, NULL) - 1; i >= 0; i--)
+        {
+          gchar *str, *tmp;
+          GtkTreeIter iter;
+
+          gtk_tree_model_iter_nth_child (model, &iter, NULL, i);
+          gtk_tree_model_get (model, &iter, column, &tmp, -1);
+          str = g_strconcat (tmp, candidate ? "," : NULL, candidate, NULL);
+          g_free (tmp);
+          g_free (candidate);
+          candidate = str;
+        }
 #else /* not GTK_CHECK_VERSION(2,4,0) */
-  glist = gtk_container_get_children
+      GList *glist;
+
+      glist = gtk_container_get_children
                                 (GTK_CONTAINER (GTK_COMBO (charset)->list));
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
-    {
-      gchar *str;
+      for (glist = g_list_first (glist);
+                            glist; glist = g_list_delete_link (glist, glist))
+        {
+          const gchar *text;
 
-      str = g_strconcat (gtk_label_get_text (GTK_LABEL (gtk_bin_get_child
-            (GTK_BIN (g_list_nth_data (glist, i))))), ",", candidate, NULL);
-      g_free (candidate);
-      candidate = str;
-    }
-   g_list_free (glist);
+          text = gtk_label_get_text
+                    (GTK_LABEL (gtk_bin_get_child (GTK_BIN (glist->data))));
+          if (candidate)
+            {
+              gchar *str;
+
+              str = g_strconcat (candidate, ",", text, NULL);
+              g_free (candidate);
+              candidate = str;
+            }
+          else
+            {
+              candidate = g_strdup (text);
+            }
+        }
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
-   return candidate;
+    }
+  return candidate;
 }
 
 
@@ -271,13 +266,14 @@ orz_charset_get_candidate (OrzCharset *charset)
     candidate,文字符号化方式の候補
           RET,TRUE:正常終了,FALSE:エラー                                    */
 gboolean
-orz_charset_add_candidate (OrzCharset *charset,
-                            const gchar *candidate)
+orz_charset_add_candidate (OrzCharset  *charset,
+                           const gchar *candidate)
 {
   gboolean result = TRUE;
   gchar **str;
   gint i;
 #if GTK_CHECK_VERSION(2,4,0)
+  gint column;
   GtkListStore *store;
 #endif /* GTK_CHECK_VERSION(2,4,0) */
 
@@ -286,6 +282,15 @@ orz_charset_add_candidate (OrzCharset *charset,
   str = g_strsplit (candidate, ",", 0);
 #if GTK_CHECK_VERSION(2,4,0)
   store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (charset)));
+# if GTK_CHECK_VERSION(2,24,0)
+  column = gtk_combo_box_get_entry_text_column (GTK_COMBO_BOX (charset));
+# else /* not GTK_CHECK_VERSION(2,24,0) */
+  for (column = gtk_tree_model_get_n_columns (GTK_TREE_MODEL (store)) - 1;
+                                                        column > 0; column--)
+    if (gtk_tree_model_get_column_type (GTK_TREE_MODEL (store), column)
+                                                            == G_TYPE_STRING)
+      break;
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
 #endif /* GTK_CHECK_VERSION(2,4,0) */
   for (i = 0; str[i]; i++)
     if (charuty_is_valid (str[i]))
@@ -303,7 +308,8 @@ orz_charset_add_candidate (OrzCharset *charset,
 
             gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store),
                                                             &iter, NULL, j);
-            gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 0, &tmp, -1);
+            gtk_tree_model_get (GTK_TREE_MODEL (store),
+                                                    &iter, column, &tmp, -1);
             cmp = g_ascii_strcasecmp (tmp, str[i]);
             g_free (tmp);
             if (cmp == 0)
@@ -315,7 +321,7 @@ orz_charset_add_candidate (OrzCharset *charset,
               break;
           }
         gtk_list_store_insert (store, &iter, j + 1);
-        gtk_list_store_set (store, &iter, 0, str[i], -1);
+        gtk_list_store_set (store, &iter, column, str[i], -1);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
         GList *glist;
         GtkWidget *item;
@@ -341,8 +347,10 @@ orz_charset_add_candidate (OrzCharset *charset,
         g_list_free (glist);
         item = gtk_list_item_new_with_label (str[i]);
         gtk_widget_show (item);
+        glist = g_list_append (NULL, item);
         gtk_list_insert_items (GTK_LIST (GTK_COMBO (charset)->list),
-                                            g_list_append (NULL, item), j + 1);
+                                                                glist, j + 1);
+        g_list_free (glist);
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
       }
     else
@@ -366,6 +374,7 @@ orz_charset_remove_candidate (OrzCharset *charset,
   gchar **str;
   gint i;
 #if GTK_CHECK_VERSION(2,4,0)
+  gint column;
   GtkListStore *store;
 #endif /* GTK_CHECK_VERSION(2,4,0) */
 
@@ -373,6 +382,15 @@ orz_charset_remove_candidate (OrzCharset *charset,
     return FALSE;
 #if GTK_CHECK_VERSION(2,4,0)
   store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (charset)));
+# if GTK_CHECK_VERSION(2,24,0)
+  column = gtk_combo_box_get_entry_text_column (GTK_COMBO_BOX (charset));
+# else /* not GTK_CHECK_VERSION(2,24,0) */
+  for (column = gtk_tree_model_get_n_columns (GTK_TREE_MODEL (store)) - 1;
+                                                        column > 0; column--)
+    if (gtk_tree_model_get_column_type (GTK_TREE_MODEL (store), column)
+                                                            == G_TYPE_STRING)
+      break;
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
 #endif /* GTK_CHECK_VERSION(2,4,0) */
   if (candidate)
     {
@@ -391,7 +409,8 @@ orz_charset_remove_candidate (OrzCharset *charset,
 
               gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store),
                                                             &iter, NULL, j);
-              gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 0, &tmp, -1);
+              gtk_tree_model_get (GTK_TREE_MODEL (store),
+                                                    &iter, column, &tmp, -1);
               cmp = g_ascii_strcasecmp (tmp, str[i]);
               g_free (tmp);
               if (cmp == 0)
