@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -267,45 +267,73 @@ edit_dialog (MmaidWindow *mmaid)
     return;
   /* ja:コンボボックス */
 #if GTK_CHECK_VERSION(2,4,0)
+# if GTK_CHECK_VERSION(2,24,0)
+  combo_title = gtk_combo_box_text_new_with_entry ();
+  combo_artist = gtk_combo_box_text_new_with_entry ();
+  combo_album = gtk_combo_box_text_new_with_entry ();
+  combo_comment = gtk_combo_box_text_new_with_entry ();
+  combo_genre = gtk_combo_box_new_with_entry ();
+# else /* not GTK_CHECK_VERSION(2,24,0) */
   combo_title = gtk_combo_box_entry_new_text ();
   combo_artist = gtk_combo_box_entry_new_text ();
   combo_album = gtk_combo_box_entry_new_text ();
   combo_comment = gtk_combo_box_entry_new_text ();
   combo_genre = gtk_combo_box_entry_new_text ();
-  for (glist_title = g_list_first (glist_title); glist_title;
-            glist_title = g_list_delete_link (glist_title, glist_title))
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
+  while ((glist_title = g_list_first (glist_title)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_title),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_title),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_title->data);
       g_free (glist_title->data);
+      glist_title = g_list_delete_link (glist_title, glist_title);
     }
-  for (glist_artist = g_list_first (glist_artist); glist_artist;
-            glist_artist = g_list_delete_link (glist_artist, glist_artist))
+  while ((glist_artist = g_list_first (glist_artist)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_artist),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_artist),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_artist->data);
       g_free (glist_artist->data);
+      glist_artist = g_list_delete_link (glist_artist, glist_artist);
     }
-  for (glist_album = g_list_first (glist_album); glist_album;
-            glist_album = g_list_delete_link (glist_album, glist_album))
+  while ((glist_album = g_list_first (glist_album)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_album),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_album),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_album->data);
       g_free (glist_album->data);
+      glist_album = g_list_delete_link (glist_album, glist_album);
     }
-  for (glist_comment = g_list_first (glist_comment); glist_comment;
-            glist_comment = g_list_delete_link (glist_comment, glist_comment))
+  while ((glist_comment = g_list_first (glist_comment)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_comment),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_comment),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_comment->data);
       g_free (glist_comment->data);
+      glist_comment = g_list_delete_link (glist_comment, glist_comment);
     }
-  for (glist_genre = g_list_first (glist_genre); glist_genre;
-            glist_genre = g_list_delete_link (glist_genre, glist_genre))
+  while ((glist_genre = g_list_first (glist_genre)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_genre),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_genre),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_genre->data);
       g_free (glist_genre->data);
+      glist_genre = g_list_delete_link (glist_genre, glist_genre);
     }
 #else /* not GTK_CHECK_VERSION(2,4,0) */
   combo_title = gtk_combo_new ();
@@ -638,7 +666,8 @@ edit_dialog (MmaidWindow *mmaid)
                                                     GTK_FILL, GTK_FILL, 0, 0);
   gtk_table_attach_defaults (GTK_TABLE (table),
                                             editdlg.charset, 3, 4, i, i + 1);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (editdlg.dialog)->vbox), table);
+  gtk_container_add (GTK_CONTAINER
+        (gtk_dialog_get_content_area (GTK_DIALOG (editdlg.dialog))), table);
 
   /* ja:表示 */
   gtk_dialog_set_response_sensitive (GTK_DIALOG (editdlg.dialog),
