@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -582,7 +582,7 @@ file_open_list (const gchar *file,
                 MmaidWindow *mmaid)
 {
   gchar *buf = NULL, *dir, **ary, **charset;
-  gint i, length, plist = 0;
+  gint i, plist = 0;
   gssize fsize;
   FileIO *fio;
   GList *glist = NULL;
@@ -651,8 +651,7 @@ file_open_list (const gchar *file,
           }
       if (!ary[j])
         break;
-      for (j = g_list_length (glist) - 1; j >= 0; j--)
-        g_free (g_list_nth_data (glist, j));
+      g_list_find_custom (glist, (GFunc)g_free, NULL);
       g_list_free (glist);
       glist = NULL;
     }
@@ -660,24 +659,21 @@ file_open_list (const gchar *file,
   g_strfreev (charset);
   g_strfreev (ary);
   g_free (dir);
-  length = g_list_length (glist);
-  if (length > 0)
+  if (g_list_length (glist) > 0)
     mmaid->playlist = plist;
-  for (i = 0; i < length; i++)
+  for (glist = g_list_first (glist);
+                            glist; glist = g_list_delete_link (glist, glist))
     {
-      gchar *path;
       ID3Tag *id3tag;
 
       while (gtk_events_pending ())
         gtk_main_iteration ();
-      path = g_list_nth_data (glist, i);
       id3tag = g_malloc0 (sizeof (ID3Tag));
-      id3tag->file  = fileio_get_full_path (path);
+      id3tag->file  = fileio_get_full_path (glist->data);
       id3tag->track = -1;
       mm_list_append (MM_LIST (mmaid->list), id3tag);
-      g_free (path);
+      g_free (glist->data);
     }
-  g_list_free (glist);
   /* ja:終了処理 */
   gtk_grab_remove (dialog);
   gtk_widget_destroy (dialog);
@@ -1030,7 +1026,7 @@ file_save_db (const gchar *file,
           dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                             GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
-                            text);
+                            "%s", text);
           g_free (text);
           g_signal_connect (G_OBJECT (dialog), "key-press-event",
                                     G_CALLBACK (misc_dialog_key_press), NULL);
