@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 */
 #include "mm_dialog.h"
 #include "mm_list.h"
-#include <gdk/gdkkeysyms.h>
 
 
 enum
@@ -54,21 +53,21 @@ mm_dialog_class_init (MmDialogClass *klass)
                                 G_TYPE_NONE, 0);
 
   binding_set = gtk_binding_set_by_class (klass);
-  gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0, "escape", 0);
+  gtk_binding_entry_add_signal (binding_set, GDK_KEY_Escape, 0, "escape", 0);
 }
 
 
 static void
 mm_dialog_escape (MmDialog *dialog)
 {
-  if (GTK_WIDGET_VISIBLE (dialog->abort_button)
-                            && GTK_WIDGET_SENSITIVE (dialog->abort_button))
+  if (gtk_widget_get_visible (dialog->abort_button)
+                        && gtk_widget_get_sensitive (dialog->abort_button))
     g_signal_emit_by_name (G_OBJECT (dialog->abort_button), "clicked");
-  else if (GTK_WIDGET_VISIBLE (dialog->cancel_button)
-                            && GTK_WIDGET_SENSITIVE (dialog->cancel_button))
+  else if (gtk_widget_get_visible (dialog->cancel_button)
+                        && gtk_widget_get_sensitive (dialog->cancel_button))
     g_signal_emit_by_name (G_OBJECT (dialog->cancel_button), "clicked");
-  else if (GTK_WIDGET_VISIBLE (dialog->ok_button)
-                            && GTK_WIDGET_SENSITIVE (dialog->ok_button))
+  else if (gtk_widget_get_visible (dialog->ok_button)
+                        && gtk_widget_get_sensitive (dialog->ok_button))
     g_signal_emit_by_name (G_OBJECT (dialog->ok_button), "clicked");
 }
 
@@ -79,16 +78,16 @@ mm_dialog_delete (GtkWidget *widget,
                   GdkEvent  *event,
                   gpointer   user_data)
 {
-  if (GTK_WIDGET_VISIBLE (MM_DIALOG (widget)->abort_button)
-                && GTK_WIDGET_SENSITIVE (MM_DIALOG (widget)->abort_button))
+  if (gtk_widget_get_visible (MM_DIALOG (widget)->abort_button)
+            && gtk_widget_get_sensitive (MM_DIALOG (widget)->abort_button))
     g_signal_emit_by_name
                     (G_OBJECT (MM_DIALOG (widget)->abort_button), "clicked");
-  else if (GTK_WIDGET_VISIBLE (MM_DIALOG (widget)->cancel_button)
-                && GTK_WIDGET_SENSITIVE (MM_DIALOG (widget)->cancel_button))
+  else if (gtk_widget_get_visible (MM_DIALOG (widget)->cancel_button)
+            && gtk_widget_get_sensitive (MM_DIALOG (widget)->cancel_button))
     g_signal_emit_by_name
                     (G_OBJECT (MM_DIALOG (widget)->cancel_button), "clicked");
-  else if (GTK_WIDGET_VISIBLE (MM_DIALOG (widget)->ok_button)
-                && GTK_WIDGET_SENSITIVE (MM_DIALOG (widget)->ok_button))
+  else if (gtk_widget_get_visible (MM_DIALOG (widget)->ok_button)
+            && gtk_widget_get_sensitive (MM_DIALOG (widget)->ok_button))
     g_signal_emit_by_name
                     (G_OBJECT (MM_DIALOG (widget)->ok_button), "clicked");
   else
@@ -130,7 +129,8 @@ mm_dialog_init (MmDialog *dialog)
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_set_border_width (GTK_CONTAINER (scroll), SPACING);
   /* ja:フレームとボックス */
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), scroll);
+  gtk_container_add (GTK_CONTAINER
+                (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), scroll);
 
   /* ja:表示 */
   gtk_widget_show_all (scroll);
