@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2010 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #define __MMAID_H__
 
 
-#include "gcommon.h"
+#include "misc/gcommon.h"
 #ifdef G_OS_WIN32
 # include <tchar.h>
 # include <windows.h>
@@ -30,7 +30,7 @@ G_BEGIN_DECLS
 
 
 #define MMAID_VERSION_NOTICE "Melody maid "VERSION" ("BUILD_ENVIRONMENT")"
-#define MMAID_COPYRIGHT_NOTICE "Copyright (C) 1998-2010 Kazuki Iwamoto"
+#define MMAID_COPYRIGHT_NOTICE "Copyright (C) 1998-2011 Kazuki Iwamoto"
 
 
 typedef struct _MmaidWindow
@@ -76,14 +76,10 @@ extern gint def_width, def_height;      /* ja:デフォルトのウインドウ
 extern gint license;                    /* ja:ライセンス */
 extern gint playlist;                   /* ja:プレイリスト */
 extern GObject *history;                /* ja:ファイルの履歴 */
-extern GdkAtom atom_targets;            /* ja:ターゲット */
 extern GdkAtom atom_id3tag;             /* ja:ID3タグ */
 extern GtkPositionType n_pos;           /* ja:ノートブックの位置 */
 extern GtkWidget *window;               /* ja:ウインドウ */
 extern GtkWidget *mdi;                  /* en:MDI */
-#ifdef G_OS_WIN32
-extern UINT uFormat;                    /* ja:クリップボード */
-#endif /* G_OS_WIN32 */
 
 
 G_END_DECLS
