@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -74,156 +74,57 @@ signal_drag_data_received (GtkWidget        *widget,
                            GdkDragContext   *context,
                            gint              x,
                            gint              y,
-                           GtkSelectionData *selection_data,
+                           GtkSelectionData *data,
                            guint             info,
                            guint             time)
 {
-  gchar *file, **files;
-  gint i;
-  MmaidWindow *mmaid;
+  gchar **files;
 
-  i = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
-  mmaid = i >= 0 ? orz_mdi_get_data (ORZ_MDI (mdi), i) : NULL;
-  files = g_strsplit ((const gchar *)selection_data->data, "\r\n", 0);
-  for (i = 0; files[i]; i++)
+  files = g_strsplit ((const gchar *)gtk_selection_data_get_data (data),
+                                                                    "\r\n", 0);
+  if (files)
     {
-      file = g_filename_from_uri (files[i], NULL, NULL);
-      if (file)
-        {
-          switch (file_is_type (file))
-            {
-              case FILE_KIND_ERROR:
-              case FILE_KIND_DB:
-              case FILE_KIND_LIST:
-                mmaid = file_open_edit (file);
-                break;
-              case FILE_KIND_UNKNOWN:
-              case FILE_KIND_TAG:
-                {
-                  ID3Tag *id3tag;
+      gint i;
+      MmaidWindow *mmaid;
 
-                  id3tag = file_open_id3tag (file, charset_list);
-                  if (id3tag)
+      i = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+      mmaid = i >= 0 ? orz_mdi_get_data (ORZ_MDI (mdi), i) : NULL;
+      for (i = 0; files[i]; i++)
+        {
+          gchar *file;
+
+          file = g_filename_from_uri (files[i], NULL, NULL);
+          if (file)
+            {
+              switch (file_is_type (file))
+                {
+                  case FILE_KIND_ERROR:
+                  case FILE_KIND_DB:
+                  case FILE_KIND_LIST:
+                    mmaid = file_open_edit (file);
+                    break;
+                  case FILE_KIND_UNKNOWN:
+                  case FILE_KIND_TAG:
                     {
-                      if (!mmaid || mm_list_search (MM_LIST (mmaid->list),
+                      ID3Tag *id3tag;
+
+                      id3tag = file_open_id3tag (file, charset_list);
+                      if (id3tag)
+                        {
+                          if (!mmaid || mm_list_search (MM_LIST (mmaid->list),
                                         MM_LIST_FILE, id3tag->file, -1) >= 0)
-                        mmaid = file_open_edit (NULL);
-                      mm_list_append (MM_LIST (mmaid->list), id3tag);
-                      orz_mdi_set_edited (ORZ_MDI (mdi),
+                            mmaid = file_open_edit (NULL);
+                          mm_list_append (MM_LIST (mmaid->list), id3tag);
+                          orz_mdi_set_edited (ORZ_MDI (mdi),
                             orz_mdi_get_page_from_data (ORZ_MDI (mdi), mmaid),
                                                                         TRUE);
+                        }
                     }
                 }
+              g_free (file);
             }
-          g_free (file);
-        }
-     }
-  g_strfreev (files);
-}
-
-
-gboolean
-signal_selection_clear (GtkWidget         *widget,
-                        GdkEventSelection *event,
-                        gpointer           user_data)
-{
-  if (event->selection == GDK_SELECTION_CLIPBOARD)
-    {
-      /* ja:クリップボード */
-      g_free (clipboard_id3tag);
-      clipboard_id3tag = NULL;
-    }
-  return TRUE;
-}
-
-
-void
-signal_selection_get (GtkWidget        *widget,
-                      GtkSelectionData *data,
-                      guint             info,
-                      guint             time,
-                      gpointer          user_data)
-{
-  /* ja:クリップボード */
-  if (data->selection == GDK_SELECTION_CLIPBOARD
-                                && clipboard_id3tag && info == TARGET_MMAID)
-    gtk_selection_data_set (data, atom_id3tag, 8,
-                    (guchar *)clipboard_id3tag, g_strlen (clipboard_id3tag));
-}
-
-
-void
-signal_selection_received (GtkWidget        *widget,
-                           GtkSelectionData *data,
-                           guint             time,
-                           gpointer          user_data)
-{
-  /* ja:クリップボード */
-  if (data->selection == GDK_SELECTION_CLIPBOARD && data->length >= 0)
-    {
-      if (data->type == GDK_SELECTION_TYPE_ATOM)
-        {
-          gint i, count;
-          GdkAtom *atoms;
-
-          atoms = (GdkAtom *)data->data;
-          count = data->length / sizeof (GdkAtom);
-          for (i = 0; i < count; i++)
-            if (atoms[i] == atom_id3tag)
-              break;
-          if (i < count)
-            gtk_selection_convert (widget, data->selection, atoms[i],
-                                                            GDK_CURRENT_TIME);
-        }
-      else if (data->type == atom_id3tag)
-        {
-          gchar **tag;
-          gint i, n, c;
-          MmaidWindow *mmaid;
-
-          /* ja:貼り付け */
-          mmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
-          mm_list_get_cursor (MM_LIST (mmaid->list), &n, &c);
-          mm_list_unselect_all (MM_LIST (mmaid->list));
-          tag = g_strsplit ((const gchar *)data->data, "\n", 0);
-          for (i = 0; tag[i]; i++)
-            {
-              gchar **a;
-              gint j, s;
-              ID3Tag *id3tag;
-
-              id3tag = g_malloc (sizeof (ID3Tag));
-              a = g_strsplit (tag[i], "\t", 0);
-              for (j = 0; a[j]; j++);
-              if (j == 9)
-                {
-                  id3tag->file    = *a[0] != '\0' ? g_strdup (a[0]) : NULL;
-                  id3tag->title   = *a[1] != '\0' ? g_strdup (a[1]) : NULL;
-                  id3tag->artist  = *a[2] != '\0' ? g_strdup (a[2]) : NULL;
-                  id3tag->album   = *a[3] != '\0' ? g_strdup (a[3]) : NULL;
-                  id3tag->year    = *a[4] != '\0' ? g_strdup (a[4]) : NULL;
-                  id3tag->comment = *a[5] != '\0' ? g_strdup (a[5]) : NULL;
-                  misc_str_to_val (&id3tag->track, a[6], 10, TRUE);
-                  id3tag->genre   = *a[7] != '\0' ? g_strdup (a[7]) : NULL;
-                  id3tag->charset = *a[8] != '\0' ? g_strdup (a[8]) : NULL;
-                  s = mm_list_search (MM_LIST (mmaid->list),
-                                            MM_LIST_FILE, id3tag->file, -1);
-                  if (s >= 0)
-                    mm_list_change (MM_LIST (mmaid->list), s,
-                                                MM_LIST_ID3TAG, id3tag, -1);
-                  else if (n >= 0)
-                    n = mm_list_insert (MM_LIST (mmaid->list), n, id3tag);
-                  else
-                    n = mm_list_append (MM_LIST (mmaid->list), id3tag);
-                  mm_list_select (MM_LIST (mmaid->list), n);
-                }
-              g_strfreev (a);
-            }
-          g_strfreev (tag);
-          mm_list_set_cursor (MM_LIST (mmaid->list), n, c);
-          orz_mdi_set_edited (ORZ_MDI (mdi),
-                    orz_mdi_get_page_from_data (ORZ_MDI (mdi), mmaid), TRUE);
-        }
+         }
+      g_strfreev (files);
     }
 }
 
@@ -307,6 +208,89 @@ signal_selection_recv (GObject   *object,
 }
 
 
+/******************************************************************************
+*                                                                             *
+* ja:シグナル/イベント関数群(クリップボード)                                  *
+*                                                                             *
+******************************************************************************/
+void
+signal_clipboard_get (GtkClipboard     *clipboard,
+                      GtkSelectionData *data,
+                      guint             info,
+                      gpointer          user_data)
+{
+  gtk_selection_data_set (data, atom_id3tag, 8,
+                                    (guchar *)user_data, g_strlen (user_data));
+}
+
+
+void
+signal_clipboard_clear (GtkClipboard *clipboard,
+                        gpointer      user_data)
+{
+  g_free (user_data);
+}
+
+
+void
+signal_clipboard_received (GtkClipboard     *clipboard,
+                           GtkSelectionData *data,
+                           gpointer          user_data)
+{
+  if (gtk_selection_data_get_length (data) >= 0
+                    && gtk_selection_data_get_data_type (data) == atom_id3tag)
+    {
+      gchar **tag;
+      gint i, n, c;
+      MmaidWindow *mmaid;
+
+      /* ja:貼り付け */
+      mmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
+      mm_list_get_cursor (MM_LIST (mmaid->list), &n, &c);
+      mm_list_unselect_all (MM_LIST (mmaid->list));
+      tag = g_strsplit ((const gchar *)gtk_selection_data_get_data (data),
+                                                                    "\n", 0);
+      for (i = 0; tag[i]; i++)
+        {
+          gchar **a;
+          gint j, s;
+          ID3Tag *id3tag;
+
+          id3tag = g_malloc (sizeof (ID3Tag));
+          a = g_strsplit (tag[i], "\t", 0);
+          for (j = 0; a[j]; j++);
+          if (j == 9)
+            {
+              id3tag->file    = *a[0] != '\0' ? g_strdup (a[0]) : NULL;
+              id3tag->title   = *a[1] != '\0' ? g_strdup (a[1]) : NULL;
+              id3tag->artist  = *a[2] != '\0' ? g_strdup (a[2]) : NULL;
+              id3tag->album   = *a[3] != '\0' ? g_strdup (a[3]) : NULL;
+              id3tag->year    = *a[4] != '\0' ? g_strdup (a[4]) : NULL;
+              id3tag->comment = *a[5] != '\0' ? g_strdup (a[5]) : NULL;
+              misc_str_to_val (&id3tag->track, a[6], 10, TRUE);
+              id3tag->genre   = *a[7] != '\0' ? g_strdup (a[7]) : NULL;
+              id3tag->charset = *a[8] != '\0' ? g_strdup (a[8]) : NULL;
+              s = mm_list_search (MM_LIST (mmaid->list),
+                                            MM_LIST_FILE, id3tag->file, -1);
+              if (s >= 0)
+                mm_list_change (MM_LIST (mmaid->list), s,
+                                                MM_LIST_ID3TAG, id3tag, -1);
+              else if (n >= 0)
+                n = mm_list_insert (MM_LIST (mmaid->list), n, id3tag);
+              else
+                n = mm_list_append (MM_LIST (mmaid->list), id3tag);
+              mm_list_select (MM_LIST (mmaid->list), n);
+            }
+          g_strfreev (a);
+        }
+      g_strfreev (tag);
+      mm_list_set_cursor (MM_LIST (mmaid->list), n, c);
+      orz_mdi_set_edited (ORZ_MDI (mdi),
+                    orz_mdi_get_page_from_data (ORZ_MDI (mdi), mmaid), TRUE);
+    }
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:シグナル/イベント関数群(メイン)                                          *
